// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "daeMain.h"
#include "utMath.h"


struct Vertex
{
	Vec3f			storedPos, pos;
	Vec3f			storedNormal, normal, tangent, bitangent;
	Vec3f			texCoords[4];
	unsigned int	joints[4];
	float			weights[4];	
	
	// Temporary
	int				posIndex;


	Vertex()
	{
		joints[0] = 0; joints[1] = 0; joints[2] = 0; joints[3] = 0;
		weights[0] = 1; weights[1] = 0; weights[2] = 0; weights[3] = 0;
	}
};


struct TriGroup
{
	unsigned int	first, count;
	unsigned int	vertRStart, vertREnd;
	string			matName;
};


struct SceneNode
{
	char					name[256];
	Matrix4f				matRel, matAbs;
	DaeNode					*daeNode;
	SceneNode				*parent;
	vector< SceneNode * >	children;

	// Animation
	vector< Matrix4f >		frames;			// Relative transformation for every frame

	SceneNode()
	{
		daeNode = 0x0;
		parent = 0x0;
	}
};


struct Mesh : public SceneNode
{
	vector< TriGroup >		triGroups;
	
	Mesh()
	{
		parent = 0x0;
	}
};


struct Joint : public SceneNode
{
	unsigned int			index;
	Matrix4f				invBindMat;
	bool					used;

	// Temporary
	Matrix4f				daeInvBindMat;

	Joint()
	{
		used = false;
	}
};


struct MorphDiff
{
	unsigned int			vertIndex;
	Vec3f					posDiff;
	Vec3f					normDiff, tanDiff, bitanDiff;
};


struct MorphTarget
{
	char					name[256];
	vector< MorphDiff >		diffs;
};


class Converter
{
private:

	vector< Vertex >		_vertices;
	vector< unsigned int >	_indices;
	vector< Mesh * >		_meshes;
	vector< Joint * >		_joints;
	vector< MorphTarget >	_morphTargets;

	unsigned int			_numAnimFrames;

	void calcTangentSpaceBasis( vector< Vertex > &vertices );
	bool writeGeometry( string name );
	void writeSGJoint( Joint *joint, unsigned int depth, ofstream &outf );
	void writeSGMesh( string modelName, Mesh *mesh, unsigned int depth, ofstream &outf );
	bool writeSceneGraph( string name );
	void writeAnimFrames( SceneNode &node, FILE *f );

public:

	Converter();
	~Converter();
	
	bool convertModel( SimpleColladaFile &collada, bool optimize );
	bool saveModel( string name );
	
	bool writeMaterials( SimpleColladaFile &collada, string name, string defShader );
	bool hasAnimation();
	bool writeAnimation( string name );
};
