// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "tinyxml.h"
#include "utils.h"
#include <string>
#include <vector>
using namespace std;


struct DaeSource
{
	string				id;
	vector< float >		floatArray;
	vector< string >	stringArray;
	unsigned int		elemsPerEntry;


	bool parse( TiXmlElement *sourceNode )
	{
		bool isFloatArray = true;
		elemsPerEntry = 1;
		
		// Check accessor
		TiXmlElement *node1 = sourceNode->FirstChildElement( "technique_common" );
		if( node1 == 0x0 ) return false;
		node1 = node1->FirstChildElement( "accessor" );
		if( node1 == 0x0 ) return false;
		
		int numEntries = atoi( node1->Attribute( "count" ) );
		if( numEntries == 0 ) return false;
		
		id = sourceNode->Attribute( "id" );

		node1 = sourceNode->FirstChildElement( "float_array" );
		if( node1 == 0x0 )
		{	
			node1 = sourceNode->FirstChildElement( "Name_array" );
			if( node1 == 0x0 ) node1 = sourceNode->FirstChildElement( "IDREF_array" );
			if( node1 == 0x0 ) return false;
			isFloatArray = false;
		}
		
		int count = atoi( node1->Attribute( "count" ) );
		elemsPerEntry = count / numEntries;

		unsigned int pos = 0;
		char *s = (char *)node1->GetText();	
		for( int i = 0; i < count; ++i )
		{
			if( isFloatArray )
			{
				float f;
				parseFloat( s, pos, f );
				floatArray.push_back( f );
			}
			else
			{
				string name;
				parseString( s, pos, name );
				stringArray.push_back( name );
			}
		}

		return true;
	}
};
