// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "tinyxml.h"
#include "daeLibImages.h"
#include <string>
#include <vector>
using namespace std;


struct DaeEffect
{
	string		id;
	string		name;
	string		diffuseMapId;
	DaeImage	*diffuseMap;


	DaeEffect()
	{
		diffuseMap = 0x0;
	}
	

	bool parse( TiXmlElement *effectNode )
	{
		id = effectNode->Attribute( "id" );
		if( effectNode->Attribute( "name" ) != 0x0 )
			name = effectNode->Attribute( "name" );

		TiXmlElement *node1 = effectNode->FirstChildElement( "profile_COMMON" );
		if( node1 == 0x0 ) return true;

		TiXmlElement *node2 = node1->FirstChildElement( "technique" );
		if( node2 == 0x0 ) return true;

		TiXmlElement *node3 = node2->FirstChildElement( "phong" );
		if( node3 == 0x0 ) node3 = node2->FirstChildElement( "blinn" );
		if( node3 == 0x0 ) node3 = node2->FirstChildElement( "lambert" );
		if( node3 == 0x0 ) return true;

		TiXmlElement *node4 = node3->FirstChildElement( "diffuse" );
		if( node4 == 0x0 ) return true;

		TiXmlElement *node5 = node4->FirstChildElement( "texture" );
		if( node5 == 0x0 ) return true;

		if( node5->Attribute( "texture" ) == 0x0 ) return true;
		string samplerId = node5->Attribute( "texture" );

		// This is a hack to support files that don't completely respect the COLLADA standard
		// and use the texture image directly instead of sampler2D
		if( node1->FirstChildElement( "newparam" ) == 0x0 )
		{
			diffuseMapId = samplerId;
			return true;
		}
		
		// Find sampler
		string surfaceId;
		node2 = node1->FirstChildElement( "newparam" );
		while( node2 != 0x0 )
		{
			if( strcmp( node2->Attribute( "sid" ), samplerId.c_str() ) == 0 )
			{
				node3 = node2->FirstChildElement( "sampler2D" );
				if( node3 == 0x0 ) return true;

				node4 = node3->FirstChildElement( "source" );
				if( node4 == 0x0 ) return true;

				if( node4->GetText() != 0x0 )
					surfaceId = node4->GetText();

				break;
			}
			
			node2 = node2->NextSiblingElement( "newparam" );
		}

		// Find surface
		node2 = node1->FirstChildElement( "newparam" );
		while( node2 != 0x0 )
		{
			if( strcmp( node2->Attribute( "sid" ), surfaceId.c_str() ) == 0 )
			{
				node3 = node2->FirstChildElement( "surface" );
				if( node3 == 0x0 ) return true;
				if( node3->Attribute( "type" ) == 0x0 ) return true;

				if( strcmp( node3->Attribute( "type" ), "2D" ) == 0 )
				{
					node4 = node3->FirstChildElement( "init_from" );
					if( node4 == 0x0 ) return true;

					if( node4->GetText() != 0x0 )
						diffuseMapId = node4->GetText();
				}

				break;
			}
			
			node2 = node2->NextSiblingElement( "newparam" );
		}
		
		return true;
	}
};


struct DaeLibEffects
{
	vector< DaeEffect * >	effects;

	
	~DaeLibEffects()
	{
		for( unsigned int i = 0; i < effects.size(); ++i ) delete effects[i];
	}
	

	DaeEffect *findEffect( string id )
	{
		if( id == "" ) return 0x0;
		
		for( unsigned int i = 0; i < effects.size(); ++i )
		{
			if( effects[i]->id == id ) return effects[i];
		}

		return 0x0;
	}


	bool parse( TiXmlElement *rootNode )
	{
		TiXmlElement *node1 = rootNode->FirstChildElement( "library_effects" );
		if( node1 == 0x0 ) return true;

		TiXmlElement *node2 = node1->FirstChildElement( "effect" );
		while( node2 != 0x0 )
		{
			DaeEffect *effect = new DaeEffect();
			if( effect->parse( node2 ) ) effects.push_back( effect );
			else delete effect;

			node2 = node2->NextSiblingElement( "effect" );
		}
		
		return true;
	}
};
