// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "tinyxml.h"
#include <string>
#include <vector>
using namespace std;


struct DaeImage
{
	string		id;
	string		name;
	string		fileName;


	bool parse( TiXmlElement *imageNode )
	{
		id = imageNode->Attribute( "id" );
		name = imageNode->Attribute( "name" );
		
		if( imageNode->FirstChildElement( "init_from" ) != 0x0 )
			fileName = imageNode->FirstChildElement( "init_from" )->GetText();
		else
			return false;

		// Extract filename from path
		fileName = extractFileName( fileName, true );
		
		return true;
	}
};


struct DaeLibImages
{
	vector< DaeImage * >	images;


	~DaeLibImages()
	{
		for( unsigned int i = 0; i < images.size(); ++i ) delete images[i];
	}


	DaeImage *findImage( string id )
	{
		if( id == "" ) return 0x0;
		
		for( unsigned int i = 0; i < images.size(); ++i )
		{
			if( images[i]->id == id ) return images[i];
		}

		return 0x0;
	}

	
	bool parse( TiXmlElement *rootNode )
	{
		TiXmlElement *node1 = rootNode->FirstChildElement( "library_images" );
		if( node1 == 0x0 ) return true;

		TiXmlElement *node2 = node1->FirstChildElement( "image" );
		while( node2 != 0x0 )
		{
			DaeImage *image = new DaeImage();
			if( image->parse( node2 ) ) images.push_back( image );
			else delete image;

			node2 = node2->NextSiblingElement( "image" );
		}
		
		return true;
	}
};
