// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "tinyxml.h"
#include "daeLibEffects.h"
#include <string>
#include <vector>
using namespace std;



struct DaeMaterial
{
	string		id;
	string		name;
	string		effectId;
	DaeEffect	*effect;
	bool		used;
	

	DaeMaterial()
	{
		used = false;
		effect = 0x0;
	}
	

	bool parse( TiXmlElement *matNode )
	{
		id = matNode->Attribute( "id" );
		if( matNode->Attribute( "name" ) != 0x0 ) name = matNode->Attribute( "name" );
		
		TiXmlElement *node1 = matNode->FirstChildElement( "instance_effect" );
		if( node1 != 0x0 )
		{
			effectId = node1->Attribute( "url" );
			removeGate( effectId );
		}
		
		return true;
	}
};


struct DaeLibMaterials
{
	vector< DaeMaterial * >	materials;


	~DaeLibMaterials()
	{
		for( unsigned int i = 0; i < materials.size(); ++i ) delete materials[i];
	}


	DaeMaterial *findMaterial( string id )
	{
		if( id == "" ) return 0x0;
		
		for( unsigned int i = 0; i < materials.size(); ++i )
		{
			if( materials[i]->id == id ) return materials[i];
		}

		return 0x0;
	}


	bool parse( TiXmlElement *rootNode )
	{
		TiXmlElement *node1 = rootNode->FirstChildElement( "library_materials" );
		if( node1 == 0x0 ) return true;

		TiXmlElement *node2 = node1->FirstChildElement( "material" );
		while( node2 != 0x0 )
		{
			DaeMaterial *material = new DaeMaterial();
			if( material->parse( node2 ) ) materials.push_back( material );
			else delete material;

			node2 = node2->NextSiblingElement( "material" );
		}
		
		return true;
	}
};
