// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#include "daeMain.h"
#include "converter.h"
#include "utPlatform.h"

#ifdef PLATFORM_WIN
#	include <windows.h>
#endif


int main( int argc, char **argv )
{
	log( "Horde3D Collada Converter" );
	log( "Version 0.12.0" );
	log( "" );
	
	if( argc < 2 )
	{
		log( "Usage:" );
		log( "ColladaConv inputFile [-o outputName] [-s shaderName] [-opt]" );
		log( "" );
		log( "inputFile:      filename of the COLLADA document" );
		log( "-o outputName:  name of the output files (without extension)" );
		log( "-s shaderName:  filename of the default shader for materials" );
		log( "-opt:           optimize geometry" );
		return 0;
	}
	
	string inName = argv[1];
	string outName = extractFileName( inName, false );
	string defShader = "skinning.shader.xml";
	bool optimize = false;

	for( int i = 2; i < argc; ++i )
	{
		if( strcmp( argv[i], "-o" ) == 0 )
		{
			if( argc > i + 1 )
			{	
				outName = argv[++i];
			}
			else
			{
				log( "Invalid argument" );
				return 0;
			}
		}
		else if( strcmp( argv[i], "-s" ) == 0 )
		{
			if( argc > i + 1 )
			{
				defShader = argv[++i];
			}
			else
			{
				log( "Invalid argument" );
				return 0;
			}
		}
		else if( strcmp( argv[i], "-opt" ) == 0 )
		{
			optimize = true;
		}
		else
		{
			log( "Invalid argument" );
			return 0;
		}
	}

	SimpleColladaFile *colladaFile = new SimpleColladaFile();
	
	log( "Parsing collada file " + inName + "..." );
	if( !colladaFile->parseFile( inName ) )
	{
		return 0;
	}
	else
	{
		log( "Done." );
	}

	// Set output directory (needed when using drag&drop of input file on application)
	#ifdef PLATFORM_WIN
	SetCurrentDirectory( extractFilePath( inName ).c_str() );
	#endif
	
	// Convert
	log( "Converting data for model " + outName + "..." );
	if( optimize ) log( "Geometry optimization enabled" );
	Converter *converter = new Converter();
	converter->convertModel( *colladaFile, optimize );
	
	converter->saveModel( outName );
	log( "Done." );
	
	log( "Writing materials..." );
	converter->writeMaterials( *colladaFile, outName, defShader );
	log( "Done." );

	if( converter->hasAnimation() )
	{
		log( "Writing animation..." );
		converter->writeAnimation( outName );
		log( "Done." );
	}
	
	delete colladaFile; colladaFile = 0x0;
	delete converter; converter = 0x0;
	
	return 1;
}
