// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "egResource.h"
#include "utMath.h"


struct Frame
{
	Quaternion			rotQuat;
	Vec3f				transVec, scaleVec;
	Matrix4f			bakedTransMat;
};


struct AnimNode
{
	string				name;
	Matrix4f			firstFrameInvTrans;
	vector< Frame >		frames;
};


class AnimationResource : public Resource
{
private:

	unsigned int		_numFrames;
	vector< AnimNode >	_nodes;

public:

	static void initResType() {};
	static void releaseResType() {};
	
	AnimationResource( const string &name, unsigned int flags );
	~AnimationResource();
	
	void initDefault();
	void release();
	bool load( const char *data, unsigned int size );
	const void *getData( ResourceData::List param );

	AnimNode *findNode( const string &name );

	friend class Renderer;
	friend class ModelNode;
};
