// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#include "egCamera.h"
#include "egSingleton.h"

#include "utDebug.h"


CameraNode::CameraNode( const string &name ) : SceneNode( SceneNodeTypes::Camera, name )
{
	setupViewParams( 45.0f, 1.333333f, 0.1f, 1000.0f );
}


CameraNode::~CameraNode()
{
	if( TheRenderer::instance().getActiveCamera() == _handle )
		TheRenderer::instance().setActiveCamera( (CameraNode *)&TheSceneManager::instance().getDefCamNode() );
}


void CameraNode::onUpdate()
{
	_invTrans = _absTrans.inverted();
}


float CameraNode::getParam( CameraNodeParams::List param )
{
	switch( param )
	{
	case CameraNodeParams::LeftPlane:
		return _frustLeft;
	case CameraNodeParams::RightPlane:
		return _frustRight;
	case CameraNodeParams::BottomPlane:
		return _frustBottom;
	case CameraNodeParams::TopPlane:
		return _frustTop;
	case CameraNodeParams::NearPlane:
		return _frustNear;
	case CameraNodeParams::FarPlane:
		return _frustFar;
	default:
		return Math::NaN;
	}
}


bool CameraNode::setParam( CameraNodeParams::List param, float value )
{
	switch( param )
	{
	case CameraNodeParams::LeftPlane:
		_frustLeft = value;
		return true;
	case CameraNodeParams::RightPlane:
		_frustRight = value;
		return true;
	case CameraNodeParams::BottomPlane:
		_frustBottom = value;
		return true;
	case CameraNodeParams::TopPlane:
		_frustTop = value;
		return true;
	case CameraNodeParams::NearPlane:
		_frustNear = value;
		return true;
	case CameraNodeParams::FarPlane:
		_frustFar = value;
		return true;
	default:
		return false;
	}
}


void CameraNode::setupViewParams( float fov, float aspect, float near, float far )
{
	float ymax = near * tanf( degToRad( fov / 2 ) );
	float xmax = ymax * aspect;

	_frustLeft = -xmax;
	_frustRight = xmax;
	_frustBottom = -ymax;
	_frustTop = ymax;
	_frustNear = near;
	_frustFar = far;
}


void CameraNode::genFrustum( Frustum &frustum )
{
	update();
	
	frustum.buildViewFrustum( _absTrans, _frustLeft, _frustRight,
							  _frustBottom, _frustTop, _frustNear, _frustFar );
}
