// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "egGlobalTypes.h"
#include <time.h>
#include <string>
#include <queue>
#include <vector>
using namespace std;


const unsigned int MaxNumMessages = 128;


class EngineConfig
{
public:
	
	// Public
	bool	trilinearFiltering;
	int		anisotropyFactor;
	bool	texCompression;
	bool	loadTextures;
	bool	fastAnimation;
	bool	occlusionCulling;
	int		shadowMapSize;
	bool	debugViewMode;

	// Internal
	bool	logging;
	float	worldSize;


	EngineConfig();
	float getOption( EngineOptions::List param );
	bool setOption( EngineOptions::List param, float value );
};


struct LogMessage
{
	string			text;
	unsigned int	level;
	float			time;

	LogMessage()
	{
	}

	LogMessage( const string &text, unsigned int level, float time ) :
		text( text ), level( level ), time( time )
	{
	}
};


class EngineLog
{
protected:
	
	unsigned long			_firstTick;
	queue< LogMessage >		_messages;

	void pushMessage( const string &text, unsigned int level );

public:
	
	EngineLog();

	void writeInfo( const string &text );
	void writeWarning( const string &text );
	void writeError( const string &text );

	bool getMessage( LogMessage &msg );
};
