// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "egResource.h"
#include "egScene.h"
#include "egAnimation.h"
#include "utMath.h"


struct VertexData
// Size as multiple of 32 bytes offers best performance because of aligned memory access
{
	Vec3f			pos, normal;
	float			u0, v0;

	float			boneVec[4];		// Float faster than int for GPU
	float			weightVec[4];

	Vec3f			tangent, bitangent;
	float			u1, v1;
};


struct Joint
{
	Matrix4f		invBindMat;
};


struct MorphDiff
{
	unsigned int	vertIndex;
	Vec3f			posDiff;
	Vec3f			normDiff, tanDiff, bitanDiff;
};


struct MorphTarget
{
	string					name;
	vector< MorphDiff >		diffs;
};


class GeometryResource : public Resource
{
private:

	vector< VertexData >		_vertData;
	bool						_16BitIndices;
	vector< unsigned int >		_indices;
	vector< Vec3f >				_vertPositions;		// For getMeshData
	
	vector< Joint >				_joints;
	vector< MorphTarget >		_morphTargets;

	unsigned int				_vertBuffer, _indexBuffer;

public:

	static unsigned int defVertBuffer, defIndexBuffer;
	
	static void initResType();
	static void releaseResType();
	
	GeometryResource( const string &name, unsigned int flags );
	~GeometryResource();
	
	void initDefault();
	void release();
	bool load( const char *data, unsigned int size );

	unsigned int getVertBuffer() { return _vertBuffer; }
	unsigned int getIndexBuffer() { return _indexBuffer; }
	Matrix4f &getInvBindMat( unsigned int jointIndex ) { return _joints[jointIndex].invBindMat; }

	friend class Renderer;
	friend class ModelNode;
	friend class MeshNode;
};
