// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

static const char *versionString = "Horde3D 0.12.0";


// Types
typedef unsigned int ResHandle;
typedef unsigned int NodeHandle;

// Predefined constants
const unsigned int RootNode = 1;
const unsigned int PrimeTimeCam = 2;

// Flags
namespace ResourceFlags
{
	enum List
	{
		NoQuery = 1,
		NoCompression = 2
	};
}


namespace EngineOptions
{
	enum List
	{
		TrilinearFiltering,
		AnisotropyFactor,
		TexCompression,
		LoadTextures,
		FastAnimation,
		OcclusionCulling,
		ShadowMapSize,
		DebugViewMode
	};
}


namespace ResourceTypes
{
	const unsigned int Count = 9;
	
	enum List
	{
		SceneGraph,
		Geometry,
		Animation,
		Material,
		Code,
		Shader,
		Texture2D,
		TextureCube,
		Effect,
		Undefined = 9999
	};
}


namespace ResourceData
{
	enum List
	{
		AnimFrameCount,
		Tex2DPixelData
	};
}


namespace SceneNodeTypes
{
	enum List
	{
		Group,
		Model,
		Mesh,
		Joint,
		Light,
		Camera,
		Emitter,
		Undefined = 9999
	};
}


namespace GroupNodeParams
{
	enum List
	{
		MinDist,
		MaxDist
	};
}


namespace MeshNodeParams
{
	enum List
	{
		MaterialRes
	};
}


namespace LightNodeParams
{
	enum List
	{
		MaterialRes,
		Radius,
		FOV,
		Col_R,
		Col_G,
		Col_B,
		ShadowMapCount,
		ShadowSplitLambda,
		ShadowMapBias
	};
}


namespace CameraNodeParams
{
	enum List
	{
		LeftPlane,
		RightPlane,
		BottomPlane,
		TopPlane,
		NearPlane,
		FarPlane
	};
}


namespace EmitterNodeParams
{
	enum List
	{
		Delay,
		EmissionRate,
		SpreadAngle,
		ForceX,
		ForceY,
		ForceZ
	};
}
