// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "utMath.h"
#include "egScene.h"

class MaterialResource;
class TiXmlDocument;
class TiXmlElement;


struct ParticleChannel
{
	float	startMin, startMax;
	float	endRate;

	ParticleChannel();
	void reset();
	bool parse( TiXmlElement *node );
};


class EffectResource : public Resource
{
private:

	float			_lifeMin, _lifeMax;
	ParticleChannel	_moveVel, _rotVel;
	ParticleChannel	_size;
	ParticleChannel	_colR, _colG, _colB, _colA;

	bool raiseError( TiXmlDocument *doc );

public:
	
	static void initResType() {};
	static void releaseResType() {};
	
	EffectResource( const string &name, unsigned int flags );
	~EffectResource();
	
	void initDefault();
	void release();
	bool load( const char *data, unsigned int size );

	friend class EmitterNode;
};


struct ParticleData
{
	float			life, maxLife;
	Vec3f			dir;
	unsigned int	respawnCounter;

	// Start values
	float			moveVel0, rotVel0;
	float			size0;
	float			r0, g0, b0, a0;
};


class EmitterNode : public SceneNode
{
protected:

	BoundingBox			_localBBox;
	
	// Emitter params
	MaterialResource	*_matRes;
	EffectResource		*_effectRes;
	unsigned int		_particleCount;
	int					_respawnCount;
	float				_delay, _emissionRate, _spreadAngle;
	Vec3f				_force;

	// Emitter data
	float				_emissionAccum;

	// Particle data
	ParticleData		*_particles;
	Vec3f				*_parPositions;
	float				*_parSizesANDRotations;
	float				*_parColors;

public:
	
	EmitterNode( const string &name, unsigned int matRes, unsigned int effectRes,
				 unsigned int maxParticleCount, int respawnCount );
	~EmitterNode();

	BoundingBox *getLocalBBox() { return &_localBBox; }
	float getParam( EmitterNodeParams::List param );
	bool setParam( EmitterNodeParams::List param, float value );

	void advanceTime( float timeDelta );

	friend class Renderer;
};
