// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "egPrimitives.h"
#include "egRenderer.h"

class TiXmlDocument;
class TiXmlElement;


namespace PipelineCommands
{
	enum List
	{
		SetRenderTarget,
		BindBuffers,
		Clear,
		DrawGeometry,
		DrawOverlays,
		DrawFSQuad,
		DrawLightGeometry,
		DrawLightShapes
	};
}


class PCParam
{
public:
	virtual ~PCParam() { }
};

class PCBoolParam : public PCParam
{
protected:
	bool	_value;
public:
	PCBoolParam( bool value ) { _value = value; }
	bool get() { return _value; }
	void set( bool value ) { _value = value; }
};

class PCFloatParam : public PCParam
{
protected:
	float	_value;
public:
	PCFloatParam( float value ) { _value = value; }
	float get() { return _value; }
	void set( float value ) { _value = value; }
};

class PCStringParam : public PCParam
{
protected:
	string	_value;
public:
	PCStringParam( const string &value ) { _value = value; }
	const string &get() { return _value; }
	void set( const string &value ) { _value = value; }
};


struct PipelineCommand
{
	PipelineCommands::List	command;
	vector< void * >		refParams;		// Pointer to object
	vector< PCParam * >		valParams;		// Newly created object
	vector< unsigned int >	resParams;		// Handle to used resources


	PipelineCommand( PipelineCommands::List	command )
	{
		this->command = command;
	}
};


struct RenderTarget
{
	string						id;
	bool						hasDepthBuffer;
	unsigned int				numTexBuffers;
	RenderBufferFormats::List	format;
	float						scale;		// Scale factor * FB_Width/FB_Height
	RenderBuffer				rendBuf;


	RenderTarget()
	{
		hasDepthBuffer = false;
		numTexBuffers = 0;
	}
};


class Pipeline
{
protected:
	
	vector< RenderTarget >		_renderTargets;
	vector< PipelineCommand >	_pipelineCommands;
	vector< Overlay >			_overlays;


	void reset();
	void addRenderTarget( const string &id, bool depthBuffer, unsigned int numBuffers,
						  RenderBufferFormats::List format, float scale );
	RenderTarget *findRenderTarget( const string &name );
	bool createRenderTargets();
	void destroyRenderTargets();

	bool raiseError( TiXmlDocument *doc );
	bool parsePipelineCommands( TiXmlElement *node );
	
	void processCommands();

public:

	~Pipeline();
	
	bool init();
	bool loadConfig( const char *configData );
	void render();
	void resize( int x, int y, int width, int height );
	
	void showOverlay( const Overlay &overlay, unsigned int matRes );
};
