// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "egRendererBase.h"
#include "egPrimitives.h"
#include "egLight.h"
#include "egCamera.h"
#include <vector>
using namespace std;

class MaterialResource;
struct ShaderContext;
class TiXmlDocument;


const unsigned int ParticlesPerBatch = 64;	// Warning: The GPU must have enough registers


struct ParticleVert
{
	float	x, y, z;		// Position
	float	u, v;			// Texture coordinates
	float	cornerIndex;	// Index of corner of quad (for up/right vectors)
	float	index;			// Index in property array

	ParticleVert()
	{
	}
	
	ParticleVert( float u, float v, float cornerIndex ):
		x( 0 ), y( 0 ), z( 0 ), u( u ), v( v ), cornerIndex( cornerIndex ), index( 0 )
	{
	}
};


class Renderer : public RendererBase
{
protected:

	unsigned int				_smFBO, _smTex;
	unsigned int				_defShadowMap;
	unsigned int				_particleVBO;
	CameraNode					*_curCamera;
	LightNode					*_curLight;
	MaterialResource			*_curMatRes;
	ShaderContext				*_curShader;
	Frustum						_camFrustum, _lightFrustum;
	unsigned int				_curUpdateStamp;
	
	float						_splitPlanes[5];
	Matrix4f					_lightMats[4];

	
	void setupViewMatrices( CameraNode *cam );
	bool setMaterial( MaterialResource *matRes, const string &shaderContext );
	void drawMeshes( const string &shaderContext, const string &theClass );
	void drawParticles( const string &shaderContext, const string &theClass );
	
	Matrix4f calcLightMat( const Frustum &frustum, bool allowCropping );
	void updateShadowMap();
	void setupShadowMap();

public:
	
	Renderer();
	~Renderer();
	
	bool init();
	void resize( int x, int y, int width, int height );
	unsigned int getActiveCamera();
	void setActiveCamera( CameraNode *cam );

	unsigned int loadShader( const char *vertexShader, const char *fragmentShader, ShaderContext &sc );
	
	bool createShadowBuffer( unsigned int width, unsigned int height );
	void destroyShadowBuffer();
	
	void bindBuffers( RenderBuffer *rb );
	void clear( bool depth, bool buf0, bool buf1, bool buf2, bool buf3, float r, float g, float b, float a );
	
	void drawOverlays( const string &shaderContext, vector< Overlay > &overlays );
	void drawFSQuad( unsigned int materialResource, const string &shaderContext );
	void drawGeometry( const string &shaderContext, const string &theClass );
	void drawLightGeometry( const string &theClass );
	void drawLightShapes();
	void renderDebugView();

	unsigned int getVPWidth() { return _vpWidth; }
	unsigned int getVPHeight() { return _vpHeight; }
};
