// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "utMath.h"
#include <string>
using namespace std;


namespace RenderBufferFormats
{
	enum List
	{
		RGBA8,
		RGBA16F,
		RGBA32F
	};
}


struct RenderBuffer
{
	unsigned int	fbo;
	unsigned int	width, height;
	unsigned int	depthBuf;
	unsigned int	texBufs[4];

	RenderBuffer()
	{
		fbo = 0;
		width = 0; height = 0;
		depthBuf = 0;
		texBufs[0] = 0; texBufs[1] = 0; texBufs[2] = 0; texBufs[3] = 0;
	}
};


class RendererBase
{
protected:

	bool			_initalized;
	unsigned int	_vpX, _vpY, _vpWidth, _vpHeight;
	string			_shaderLog;
	RenderBuffer	*_curRendBuf;


	void myPerspective( float fovy, float aspect, float zNear, float zFar );

public:
	
	RendererBase();
	
	// Rendering functions
	virtual bool init();
	virtual void resize( int x, int y, int width, int height );
	
	// Vertex buffer functions
	unsigned int uploadVertices( void *data, unsigned int size, unsigned int bufId = 0 );
	unsigned int uploadIndices( void *indices, unsigned int size, unsigned int bufId = 0 );
	void unloadBuffers( unsigned int vertBufId, unsigned int idxBufId );

	// Texture map functions
	bool supportsNPOTTextures();
	unsigned int uploadTexture2D( unsigned char *buffer, int width, int height, int bpp,  bool allowCompression, unsigned int texId = 0 );
	void updateTexture2D( unsigned char *buffer, int width, int height, int bpp, unsigned int texId );
	unsigned int uploadTextureCube( unsigned char *buffer, int width, int height, int bpp, unsigned int cubeFace, bool allowCompression, unsigned int texId = 0 );
	void unloadTexture( unsigned int texId, bool cubeMap );

	// Shader functions
	virtual unsigned int loadShader( const char *vertexShader, const char *fragmentShader );
	void unloadShader( unsigned int shaderId );
	void setShader( unsigned int shaderId );
	string &getShaderLog() { return _shaderLog; }

	// Render buffer functions
	RenderBuffer createRenderBuffer( unsigned int width, unsigned int height, RenderBufferFormats::List format,
									 bool depth, unsigned int numTextures );
	void destroyRenderBuffer( RenderBuffer &rb );
	void setRenderBuffer( RenderBuffer *rb );
};
