// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include <vector>
#include <string>
#include "egGlobalTypes.h"
using namespace std;


class Resource
{
protected:

	ResourceTypes::List	_type;
	string				_name;
	unsigned int		_handle;
	bool				_loaded;
	bool				_noQuery;
	
	bool				_userRef;	// Resource was explicitly added by user
	unsigned int		_refCount;	// Number of other objects referencing to this resource

public:

	Resource( ResourceTypes::List type, const string &name, unsigned int flags );
	virtual ~Resource();
	
	virtual void initDefault();
	virtual void release();
	virtual bool load( const char *data, unsigned int size );
	void unload();
	virtual const void *getData( ResourceData::List param );
	virtual bool updateData( ResourceData::List param, const void *data, unsigned int size );

	ResourceTypes::List &getType() { return _type; }
	const string &getName() { return _name; }
	unsigned int getHandle() { return _handle; }

	void spinRefCount( bool inc )
	{
		if( inc ) ++_refCount;
		else { if( _refCount > 0 ) --_refCount; }
	}

	friend class ResourceManager;
};


class ResourceManager
{
protected:

	vector < Resource * >	_resources;
	vector < string >		_resourcePaths;

public:

	ResourceManager();
	~ResourceManager();

	bool init();

	const string getResourcePath( ResourceTypes::List type );
	void setResourcePath( ResourceTypes::List type, string path );
	
	Resource *findResource( ResourceTypes::List type, const string &name );
	unsigned int addResource( ResourceTypes::List type, const string &name, unsigned int flags, bool userCall );
	bool removeResource( unsigned int handle, bool userCall );
	void clear();
	bool loadResource( const string &name, const char *data, unsigned int size );
	const void *getResourceData( unsigned int handle, ResourceData::List param );
	bool updateResourceData( unsigned int handle, ResourceData::List param, const void *data, unsigned int size );
	string queryUnloadedResource();

	void releaseUnusedResources();

	Resource *resolveResHandle( unsigned int handle )
		{ return (handle != 0 && handle - 1 < _resources.size()) ? _resources[handle - 1] : 0x0; }

	vector < Resource * > &getResources() { return _resources; }
};

