// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "egResource.h"


class TiXmlDocument;
class TiXmlElement;


class CodeResource : public Resource
{
private:
	
	string	_code;

	void updateShaders();

public:
	
	static void initResType() {};
	static void releaseResType() {};
	
	CodeResource( const string &name, unsigned int flags );
	~CodeResource();
	
	void initDefault();
	void release();
	bool load( const char *data, unsigned int size );

	bool isLoaded() { return _loaded; }
	const string &getCode() { return _code; }

	friend class Renderer;
};


struct ShaderCodeFract
{
	CodeResource	*refCodeRes;
	string			code;


	ShaderCodeFract()
	{
		refCodeRes = 0x0;
	}
};


namespace BlendModes
{
	enum List
	{
		Replace,
		Blend,
		Add,
		AddBlended
	};
}


struct ShaderContext
{
	string						id;
	vector< ShaderCodeFract >	vertShaderFracts, fragShaderFracts;
	bool						compiled;
	unsigned int				shaderObject;
	unsigned int				lastUpdateStamp;

	// RenderConfig
	bool						writeDepth;
	BlendModes::List			blendMode;
	
	// Uniform and attribute locations
	int							uni_texs[8];
	int							uni_worldMat, uni_worldNormalMat;
	int							uni_viewer;
	int							uni_lightPos, uni_lightDir, uni_lightColor, uni_lightCosCutoff;
	int							uni_shadowSplitDists, uni_shadowMats;
	int							uni_shadowMapSize, uni_shadowBias;
	int							uni_skinMatRows;
	int							attrib_texCoords0, attrib_texCoords1;
	int							attrib_normal, attrib_tangent, attrib_bitangent;
	int							attrib_bones, attrib_weights;


	ShaderContext()
	{
		compiled = false;
		shaderObject = 0;
		lastUpdateStamp = 0;
		writeDepth = true;
		blendMode = BlendModes::Replace;
	}
};


class ShaderResource : public Resource
{
private:
	
	vector< ShaderContext >		_contexts;

	bool raiseError( TiXmlDocument *doc );
	bool parseCode( TiXmlElement *node, vector< ShaderCodeFract > &codeFracts );

public:
	
	static void initResType() {};
	static void releaseResType() {};
	
	ShaderResource( const string &name, unsigned int flags );
	~ShaderResource();
	
	void initDefault();
	void release();
	bool load( const char *data, unsigned int size );
	void compileShaders();

	ShaderContext *findContext( const string &name )
	{
		for( unsigned int i = 0; i < _contexts.size(); ++i )
			if( _contexts[i].id == name ) return &_contexts[i];
		
		return 0x0;
	}

	friend class Renderer;
};
