// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2007 Nicolas Schulz
//
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
// *************************************************************************************************

#pragma once

#include "egResource.h"


class ImageLoader
{
private:

	static unsigned int nextSmallerPowerOfTwo( unsigned int n );

public:

	static bool loadImage( const char *data, unsigned int size, bool makePOT,
						   unsigned char *&pixels, int &width, int &height, int &bpp );
};



class Texture2DResource : public Resource
{
private:
	
	int				_width, _height, _bpp;
	unsigned char	*_pixels;
	bool			_allowCompression;
	unsigned int	_texObject;

public:
	
	static unsigned int defTexObject;

	static void initResType();
	static void releaseResType();
	
	Texture2DResource( const string &name, unsigned int flags );
	~Texture2DResource();
	
	void initDefault();
	void release();
	bool load( const char *data, unsigned int size );
	bool updateData( ResourceData::List param, const void *data, unsigned int size );

	unsigned int getTexObject() { return _texObject; }

	friend class ResourceManager;
};


class TextureCubeResource : public Resource
{
private:
	
	int				_width, _height, _bpp;
	unsigned char	*_pixels;
	bool			_allowCompression;
	unsigned int	_texObject;

public:
	
	static unsigned int defTexObject;

	static void initResType();
	static void releaseResType();
	
	TextureCubeResource( const string &name, unsigned int flags );
	~TextureCubeResource();
	
	void initDefault();
	void release();
	bool load( const char *data, unsigned int size );

	unsigned int getTexObject() { return _texObject; }

	friend class ResourceManager;
};
